@@ -151,6 +151,7 @@ class geneticalgorithm2:
 
         self.f: Callable[[np.ndarray], float] = None
         self.funtimeout: float = None
+        self.set_function: Callable[[np.ndarray], np.ndarray] = None
 
         # self.dim: int = None
         self.var_bounds: List[Tuple[Union[int, float], Union[int, float]]] = None
@@ -160,6 +161,7 @@ class geneticalgorithm2:
         self.checked_reports: List[Tuple[str, Callable[[np.ndarray], None]]] = None
 
         self.population_size: int = None
+        self.progress_stream = None
 
 
         # input algorithm's parameters
@@ -194,8 +196,10 @@ class geneticalgorithm2:
         assert self.param.mutation_discrete_probability is None or can_be_prob(self.param.mutation_discrete_probability)
         self.prob_mut_discrete = self.param.mutation_discrete_probability or self.prob_mut
 
-        assert can_be_prob(self.param.crossover_probability)
-        self.prob_cross = self.param.crossover_probability
+        if self.param.crossover_probability is not None:
+            warnings.warn(
+                f"crossover_probability is deprecated and will be removed in version 7. Reason: it's old and has no sense"
+            )
 
         #############################################################
         # input function
@@ -264,8 +268,8 @@ class geneticalgorithm2:
     def _set_parents_count(self, parents_portion: float):
 
         self.parents_count = int(parents_portion * self.population_size)
-        assert self.parents_count <= self.population_size, f'parents count {self.parents_count} cannot be less than population size {self.population_size}'
-        trl= self.population_size - self.parents_count
+        assert self.parents_count <= self.population_size and self.parents_count > 1, f'parents count {self.parents_count} cannot be less than population size {self.population_size}'
+        trl = self.population_size - self.parents_count
         if trl % 2 != 0:
             self.parents_count += 1
 
@@ -350,8 +354,8 @@ class geneticalgorithm2:
         percents = round(100.0 * part, 1)
         bar = '|' * filled_len + '_' * (geneticalgorithm2.PROGRESS_BAR_LEN - filled_len)
 
-        sys.stdout.write('\r%s %s%s %s' % (bar, percents, '%', status))
-        sys.stdout.flush()
+        self.progress_stream.write('\r%s %s%s %s' % (bar, percents, '%', status))
+        self.progress_stream.flush()
 
     def __str__(self):
         return f"Genetic algorithm object with parameters {self.param}"
@@ -397,8 +401,11 @@ class geneticalgorithm2:
     def run(
         self,
         no_plot: bool = False,
-        disable_progress_bar: bool = False,
         disable_printing: bool = False,
+        progress_bar_stream: Optional[str] = 'stdout',
+
+        # deprecated
+        disable_progress_bar: bool = False,
 
         set_function: Optional[Callable[[np.ndarray], np.ndarray]] = None,
         apply_function_to_parents: bool = False,
@@ -428,7 +435,7 @@ class geneticalgorithm2:
         """
         @param no_plot <boolean> - do not plot results using matplotlib by default
         
-        @param disable_progress_bar <boolean> - do not show progress bar
+        @param progress_bar_stream -- 'stdout', 'stderr' or None to disable progress bar
                 
         @param set_function : 2D-array -> 1D-array function, which applyes to matrix of population (size (samples, dimention))
         to estimate their values
@@ -465,6 +472,12 @@ class geneticalgorithm2:
         @param seed (None/int) - random seed (None if doesn't matter)
         """
 
+        if disable_progress_bar:
+            warnings.warn(
+                f"disable_progress_bar is deprecated and will be removed in version 7, use probress_bar_stream=None to disable progress bar"
+            )
+            progress_bar_stream = None
+
         start_generation = Generation.from_object(self.dim, start_generation)
 
 
@@ -482,8 +495,17 @@ class geneticalgorithm2:
         # randomstate = np.random.default_rng(random.randint(0, 1000) if seed is None else seed)
         # self.randomstate = randomstate
 
-        show_progress = (lambda t, t2, s: self.__progress(t, t2, status=s)) if not disable_progress_bar else (lambda t, t2, s: None)
-        
+        show_progress = (lambda t, t2, s: self.__progress(t, t2, status=s)) if progress_bar_stream is not None else (lambda t, t2, s: None)
+        if progress_bar_stream is not None:
+            if progress_bar_stream == 'stdout':
+                self.progress_stream = sys.stdout
+            elif progress_bar_stream == 'stderr':
+                self.progress_stream = sys.stderr
+            else:
+                raise Exception(
+                    f"wrong value {progress_bar_stream} of progress_bar_stream, must be 'stdout'/'stderr'/None"
+                )
+
         stop_by_val = (lambda best_f: False) if stop_when_reached is None else (lambda best_f: best_f <= stop_when_reached)
 
         t = 0
@@ -502,7 +524,6 @@ class geneticalgorithm2:
 
                 mutation_prob=self.prob_mut,
                 mutation_discrete_prob=self.prob_mut_discrete,
-                crossover_prob=self.prob_cross,
                 mutation=self.real_mutation,
                 mutation_discrete=self.discrete_mutation,
                 crossover=self.crossover,
@@ -536,7 +557,6 @@ class geneticalgorithm2:
 
             self.prob_mut = data.mutation_prob
             self.prob_mut_discrete = data.mutation_discrete_prob
-            self.prob_cross = data.crossover_prob
             self.real_mutation = data.mutation
             self.discrete_mutation = data.mutation_discrete
             self.crossover = data.crossover
@@ -571,13 +591,11 @@ class geneticalgorithm2:
                 if flag:
                     set_data(data)  # update global date if there was real callback step
 
-
         start_time = time.time()
         time_is_done = (lambda: False) if time_limit_secs is None else (lambda: int(time.time() - start_time) >= time_limit_secs)
 
         ############################################################# 
         # Initial Population
-        
         self.set_function = set_function or geneticalgorithm2.default_set_function(self.f)
 
         pop_coef, initializer_func = population_initializer
@@ -769,11 +787,8 @@ class geneticalgorithm2:
         count_stagnation = 0  # iterations without progress
         reason_to_stop: Optional[str] = None
 
-        par_count_for_crossover = fast_max(2, math.floor(self.parents_count * self.prob_cross))  # at least 2 parents
         # gets indexes of 2 parents to crossover
-        get_parents_inds = (lambda: (0, random.randrange(1, par_count_for_crossover))) if studEA else (lambda: random.sample(range(par_count_for_crossover), 2))
-        if par_count_for_crossover == 2:
-            warnings.warn("selected only 2 parents for crossover!!! increase population size of parents portion or crossover probability")
+        get_parents_inds = (lambda: (0, random.randrange(1, self.parents_count))) if studEA else (lambda: random.sample(range(self.parents_count), 2))
 
         #  while no reason to stop
         while True:
@@ -821,22 +836,11 @@ class geneticalgorithm2:
                 
             # non-elit parents indexes
             new_par_inds = self.selection(scores[self.elit_count:], self.parents_count - self.elit_count).astype(np.int16) + self.elit_count
+            #new_par_inds = self.selection(scores, self.parents_count - self.elit_count).astype(np.int16)
             par_slice = slice(self.elit_count, self.parents_count)
             par[par_slice] = pop[new_par_inds].copy()
             par_scores[par_slice] = scores[new_par_inds].copy()
 
-            #
-            #  select parents for crossover
-            #
-
-            #  select random parents
-            ef_par_list = np.random.choice(
-                np.arange(self.parents_count),
-                par_count_for_crossover,
-                replace=False
-            )
-            ef_par = par[ef_par_list].copy()
-
             # New generation
             pop = np.empty((self.population_size, self.dim))
             scores = np.empty(self.population_size)
@@ -844,15 +848,16 @@ class geneticalgorithm2:
             parents_slice = slice(None, self.parents_count)
             pop[parents_slice] = par[parents_slice]
             scores[parents_slice] = par_scores[parents_slice]
-                
+
+            DO_MUTATION = self.needs_mutation
             for k in range(self.parents_count, self.population_size, 2):
                 r1, r2 = get_parents_inds()
-                pvar1 = ef_par[r1]
-                pvar2 = ef_par[r2]
+                pvar1 = par[r1]
+                pvar2 = par[r2]
                 
                 ch1, ch2 = self.crossover(pvar1, pvar2)
                 
-                if self.needs_mutation:
+                if DO_MUTATION:
                     ch1 = self.mut(ch1)
                     ch2 = self.mut_middle(ch2, pvar1, pvar2)               
 
@@ -874,8 +879,7 @@ class geneticalgorithm2:
 
             t += 1
 
-        if scores[0] < self.best_function:
-            self.best_function = scores[0]
+        self.best_function = fast_min(scores[0], self.best_function)
 
         last_generation = Generation(pop, scores)
         self.result = GAResult(last_generation)
@@ -953,13 +957,11 @@ class geneticalgorithm2:
 
         for i in self.indexes_int_mut:
             if random.random() < self.prob_mut_discrete:
-                lower, upper = self.var_bounds[i]
-                x[i] = self.discrete_mutation(x[i], lower, upper)
+                x[i] = self.discrete_mutation(x[i], *self.var_bounds[i])
 
         for i in self.indexes_float_mut:                
             if random.random() < self.prob_mut:
-                lower, upper = self.var_bounds[i]
-                x[i] = self.real_mutation(x[i], lower, upper)
+                x[i] = self.real_mutation(x[i], *self.var_bounds[i])
             
         return x
 
@@ -976,8 +978,7 @@ class geneticalgorithm2:
                 elif v1 > v2:
                     x[i] = random.randint(v2, v1)
                 else:
-                    lower, upper = self.var_bounds[i]
-                    x[i] = random.randint(lower, upper)
+                    x[i] = random.randint(*self.var_bounds[i])
                         
         for i in self.indexes_float_mut:                
             if random.random() < self.prob_mut:
@@ -985,8 +986,7 @@ class geneticalgorithm2:
                 if v1 != v2:
                     x[i] = random.uniform(v1, v2)
                 else:
-                    lower, upper = self.var_bounds[i]
-                    x[i] = random.uniform(lower, upper)
+                    x[i] = random.uniform(*self.var_bounds[i])
         return x
 
     #endregion
@@ -1004,6 +1004,16 @@ class geneticalgorithm2:
             return np.array([function_for_set(matrix[i]) for i in range(matrix.shape[0])])
         return func
 
+    @staticmethod
+    def vectorized_set_function(function_for_set: Callable[[np.ndarray], float]):
+        """
+        works like default, but faster for big populations and slower for little
+        function_for_set just applyes to each row of population
+        """
+
+        func = np.vectorize(function_for_set, signature='(n)->()')
+        return func
+
     @staticmethod
     def set_function_multiprocess(function_for_set: Callable[[np.ndarray], float], n_jobs: int = -1):
         """
